; *********************************************************** ;
;                           DIMMER                            ;
;       make the luminosity of a LED controllable by a        ;
;       potentiometer using the ADC and the PWM modules       ;
;                                                             ;
;       INFO0064 - Embedded Systems                           ;
;       Author: Kevin Hogan                                   ;
;                                                             ;
; *********************************************************** ;


    #include 	"config.inc"
    #DEFINE	value_counter	0x4C    ;value used to obtain the required delay

; Set the first variable address in the RAM to 0x00
	cblock	0x00
	endc

; That is where the MCU will start executing the program (0x00)
	org 	0x00
	nop
	goto    start           ; jump to the beginning of the code

; interrupt vector: if an interrupt occurs, the MCU jumps to this address (0x08)
; we need to jump to the code that will handle the interrupt request
	org 	0x08
	goto 	high_priority_interrupt_vector


;BEGINNING OF THE PROGRAM
start:
	call	initialisation      ; initialisation routine configuring the MCU
	goto	main_loop           ; main loop

;INITIALISATION
initialisation:

    ;configuration of the GPIO
	clrf    TRISB               ; All pins of PORTB are output
	bsf	LATB , 0	    ; RB0 = 1 while RB1..7 = 0;
	
	clrf	TRISC
	bcf	TRISC , 3	    ; RC2 = output PWM
	
	clrf	TRISC
	bsf	TRISA , 0	    ; RA0 = input ADC

    ;configuration of Timer0
    ; Timer0 ON - 16 bits mode - NO prescaling
	movlw	b'10001000'
	movwf	T0CON               ; configure Timer0 (cf. datasheet SFR T0CON)

    ;configure PWM module
	movlw	0xFF		; 
	movwf	PIR2		; set the PWM period

	movlw	0x3F		;
	movwf	CCPR1L		; 
	bcf	CCP1CON , 4	; 
	bcf	CCP1CON , 5	; set the 10-bit PWM duty cycle

	movlw	b'00000111'	; 
	movwf	T2CON		; configure Timer2

	movlw	b'00001100'	; 
	movwf	CCP1CON		; set the PWM

    ;configure A/D converter
    ; Vref+ = Vss; Vref- = Vdd; input = AN0
	movlw	b'00001110'
	movwf	ADCON1
    ; input = AN0
	movlw	b'00000000'
	movwf	ADCON0
    ; Left justified; Tacq = 0; Tad = 8 Tosc
	movlw	b'00000001'	
	movwf	ADCON2
    ; A/D converter on
	movlw	b'00000001'
	movwf	ADCON0
	
    ;set the counter
	movlw	value_counter
	movwf	counter

    ;enable interrupts
        bcf	INTCON , TMR0IF	    ; clear interrupt flag for Timer0
	bsf	INTCON , TMR0IE     ; enable interrupt for Timer0
	bcf	PIR1 , ADIF	    ; clear interrupt flag for ADC
	bsf	PIE1 , ADIE	    ; enable interrupt for ADC
	bsf	INTCON , PEIE	    ; enable peripherals interrupts	
	bsf	INTCON , GIE        ; enable interrupts

	return


	cblock
	counter	:	1
	endc

;INTERRUPT ROUTINE
; Here we have to determine what triggered the interrupt request
high_priority_interrupt_vector:

;Timer0 test
test_int_Timer_0:
	btfss	INTCON , TMR0IE     ; Is Timer0 Interrupt Enable bit set?
	goto	test_int_ADC	    ; skip if yes

	btfsc	INTCON , TMR0IF     ; Is Timer0 Interrupt Flag bit set?
	call	interrupt_Timer_0   ; skip if no

test_int_ADC:
	btfss	PIE1 , ADIE     ; Is ADC Interrupt Enable bit set?
	goto	other_test_int	; skip if yes

	btfsc	PIR1 , ADIF     ; Is ADC Interrupt Flag bit set?
	call	interrupt_ADC   ; skip if no
	
;Other tests (if any)
other_test_int:

end_of_test_int:
	retfie 	1                   ; return from the interrupt routine

;TIMER0 INTERRUPT ROUTINE
; we have to wait for value_counter interrupts before changing the value of RB0
; (value_counter has been determined during Exercice Session 2 (cf. slide 16))
interrupt_Timer_0:
	bcf	INTCON , TMR0IF		; Clear the Interrupt Flag!
	bsf	ADCON0 , GO	    ; start ADC conversion
	decfsz	counter , f         ; if counter = 0 then the next instruction is skipped else counter--
	return

	movlw	0x01
	xorwf	LATB , f            ; RB0 = !RB0

	movlw	value_counter
	movwf	counter             ; reset the value of the counter

	return

;ADC INTERRUPT ROUTINE
; the A/D converter just finished converting the voltage, we will set this value
; as the new duty cycle
interrupt_ADC:
    bcf	    PIR1 , ADIF	    ;clear the interrupt flag
    
    movff   ADRESH , CCPR1L
    
    return
;MAIN LOOP
; nothing to do but wait for interrupts
main_loop:
	nop
	goto	main_loop

	END